@@ -20,10 +20,6 @@ using namespace std;
 #include "LanguageModelling/DictLanguageModel.h"
 #include "LanguageModelling/MixtureLanguageModel.h"
 
-#ifdef JAPANESE
-#include "LanguageModelling/JapaneseLanguageModel.h"
-#endif
-
 using namespace Dasher;
 using namespace std;
 
@@ -41,11 +37,23 @@ static char THIS_FILE[] = __FILE__;
 
 // CDasherModel
 
-CDasherModel::CDasherModel(CEventHandler *pEventHandler, CSettingsStore *pSettingsStore, CDasherInterfaceBase *pDashIface, CAlphIO *pAlphIO)
+CDasherModel::CDasherModel(CEventHandler *pEventHandler, CSettingsStore *pSettingsStore, CDasherInterfaceBase *pDashIface, CAlphIO *pAlphIO, bool bGameMode, const std::string &strGameModeText)
 :CDasherComponent(pEventHandler, pSettingsStore), m_pDasherInterface(pDashIface), m_Root(0), total_nats(0.0), 
-m_pLanguageModel(NULL), m_pcAlphabet(NULL), m_pGameMode(NULL), m_Rootmin(0), m_Rootmax(0), m_Rootmin_min(0),
+m_pLanguageModel(NULL), m_pcAlphabet(NULL), m_Rootmin(0), m_Rootmax(0), m_Rootmin_min(0),
 m_Rootmax_max(0), m_dAddProb(0.0), m_dMaxRate(0.0) {
 
+  m_iTargetOffset = 0;
+
+  m_bGameMode = bGameMode;
+
+#ifdef JAPANESE
+  m_bRequireConversion = true;
+#else
+  m_bRequireConversion = false;
+#endif
+
+  SetBoolParameter(BP_CONVERSION_MODE, m_bRequireConversion);
+
   // Set max bitrate in the FrameRate class
   m_dMaxRate = GetLongParameter(LP_MAX_BITRATE) / 100.0;
   m_fr.SetMaxBitrate(m_dMaxRate);
@@ -89,11 +97,6 @@ m_Rootmax_max(0), m_dAddProb(0.0), m_dMaxRate(0.0) {
   case 3:
     m_pLanguageModel = new CMixtureLanguageModel(m_pEventHandler, m_pSettingsStore, alphabet);
     break;  
-#ifdef JAPANESE
-  case 4:
-    m_pLanguageModel = new CJapaneseLanguageModel(m_pEventHandler, m_pSettingsStore, alphabet);
-    break;
-#endif
   default:
     // If there is a bogus value for the language model ID, we'll default
     // to our trusty old PPM language model.
@@ -116,10 +119,7 @@ m_Rootmax_max(0), m_dAddProb(0.0), m_dMaxRate(0.0) {
   m_Rootmin_min = int64_min / iNormalization / 2;
   m_Rootmax_max = int64_max / iNormalization / 2;
 
-  // Initialize Game Mode object
-  m_pGameMode = new CDasherGameMode(pEventHandler, pSettingsStore, m_pDasherInterface, this);
-
-  m_pAlphabetManagerFactory = new CAlphabetManagerFactory(this, m_pLanguageModel);
+  m_pAlphabetManagerFactory = new CAlphabetManagerFactory(this, m_pLanguageModel, bGameMode, strGameModeText);
   m_pControlManagerFactory = new CControlManagerFactory(this, m_pLanguageModel);
 #ifdef JAPANESE
   m_pConversionManagerFactory = new CConversionManagerFactory(this, m_pLanguageModel);
@@ -146,11 +146,6 @@ CDasherModel::~CDasherModel() {
   delete m_pConversionManagerFactory;
 #endif
 
-  if (m_pGameMode != NULL) {
-    delete m_pGameMode;
-    m_pGameMode = NULL;
-  }
-
   m_pLanguageModel->ReleaseContext(LearnContext);
   delete m_pLanguageModel;
 
@@ -171,6 +166,9 @@ void CDasherModel::HandleEvent(Dasher::CEvent *pEvent) {
     case BP_CONTROL_MODE: // Rebuild the model if control mode is switched on/off
       RebuildAroundNode(Get_node_under_crosshair());
       break;
+    case BP_DELAY_VIEW:
+      MatchTarget(GetBoolParameter(BP_DELAY_VIEW));
+      break;
     default:
       break;
     }
@@ -199,7 +197,7 @@ void CDasherModel::Make_root(CDasherNode *whichchild)
 
   m_Root = whichchild;
 
-  while(oldroots.size() > 10) {
+  while((oldroots.size() > 10) && (!m_bRequireConversion || (oldroots[0]->GetConverted()))) {
     oldroots[0]->OrphanChild(oldroots[1]);
     delete oldroots[0];
     oldroots.pop_front();
@@ -208,6 +206,16 @@ void CDasherModel::Make_root(CDasherNode *whichchild)
   myint range = m_Rootmax - m_Rootmin;
   m_Rootmax = m_Rootmin + (range * m_Root->Hbnd()) / (int)GetLongParameter(LP_NORMALIZATION);
   m_Rootmin = m_Rootmin + (range * m_Root->Lbnd()) / (int)GetLongParameter(LP_NORMALIZATION);
+  
+  for(std::deque<SGotoItem>::iterator it(m_deGotoQueue.begin()); it != m_deGotoQueue.end(); ++it) {
+    myint r = it->iN2 - it->iN1;
+    it->iN2 = it->iN1 + (r * m_Root->Hbnd()) / (int)GetLongParameter(LP_NORMALIZATION);
+    it->iN1 = it->iN1 + (r * m_Root->Lbnd()) / (int)GetLongParameter(LP_NORMALIZATION);
+  }
+
+//   myint iTargetRange = m_iTargetMax - m_iTargetMin;
+//   m_iTargetMax = m_iTargetMin + (iTargetRange * m_Root->Hbnd()) / (int)GetLongParameter(LP_NORMALIZATION);
+//   m_iTargetMin = m_iTargetMin + (iTargetRange * m_Root->Lbnd()) / (int)GetLongParameter(LP_NORMALIZATION);
 }
 
 void CDasherModel::ClearRootQueue() {
@@ -269,6 +277,9 @@ void CDasherModel::Reparent_root(int lower, int upper) {
     
     pNewRoot = m_Root->m_pNodeManager->RebuildParent(m_Root, iGenerations);
 
+    //    std::cout << pNewRoot->m_pNodeManager << std::endl;
+    //    pNewRoot->m_bWatchDelete = true;
+
     lower = m_Root->Lbnd();
     upper = m_Root->Hbnd();
 
@@ -287,6 +298,7 @@ void CDasherModel::Reparent_root(int lower, int upper) {
   /* Determine how zoomed in we are */
 
   myint iRootWidth = m_Rootmax - m_Rootmin;
+  //  myint iTargetWidth = m_iTargetMax - m_iTargetMin;
   myint iWidth = upper - lower;
 //  double scalefactor=(m_Rootmax-m_Rootmin)/static_cast<double>(upper-lower);
 
@@ -294,25 +306,38 @@ void CDasherModel::Reparent_root(int lower, int upper) {
 
   m_Rootmax = m_Rootmax + (myint((GetLongParameter(LP_NORMALIZATION) - upper)) * iRootWidth / iWidth);
   m_Rootmin = m_Rootmin - (myint(lower) * iRootWidth / iWidth);
+  
+//   m_iTargetMax = m_iTargetMax + (myint((GetLongParameter(LP_NORMALIZATION) - upper)) * iTargetWidth / iWidth);
+//   m_iTargetMin = m_iTargetMin - (myint(lower) * iTargetWidth / iWidth);
+  
+  
 }
 
 /////////////////////////////////////////////////////////////////////////////
 
 CDasherNode *CDasherModel::Get_node_under_crosshair() {
-  return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin, m_Rootmax, GetLongParameter(LP_OX), GetLongParameter(LP_OY));
+//   if(GetBoolParameter(BP_DELAY_VIEW))
+//     return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_iTargetMin, m_iTargetMax, GetLongParameter(LP_OX), GetLongParameter(LP_OY));
+//   else
+    return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin + m_iTargetOffset, m_Rootmax + m_iTargetOffset, GetLongParameter(LP_OX), GetLongParameter(LP_OY));
 }
 
 /////////////////////////////////////////////////////////////////////////////
 
 CDasherNode *CDasherModel::Get_node_under_mouse(myint Mousex, myint Mousey) {
-  return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin, m_Rootmax, Mousex, Mousey);
+//  if(GetBoolParameter(BP_DELAY_VIEW))
+//     return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_iTargetMin, m_iTargetMax, Mousex, Mousey);
+//   else
+  return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin + m_iTargetOffset, m_Rootmax + m_iTargetOffset, Mousex, Mousey);
 }
 
 /////////////////////////////////////////////////////////////////////////////
 
 void CDasherModel::Get_string_under_mouse(const myint Mousex, const myint Mousey, vector <symbol >&str) {
-  m_Root->Get_string_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin, m_Rootmax, Mousex, Mousey, str);
-  return;
+ //  if(GetBoolParameter(BP_DELAY_VIEW))
+//     m_Root->Get_string_under(GetLongParameter(LP_NORMALIZATION), m_iTargetMin, m_iTargetMax, Mousex, Mousey, str);
+//   else
+    m_Root->Get_string_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin + m_iTargetOffset, m_Rootmax + m_iTargetOffset, Mousex, Mousey, str);
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -381,6 +406,11 @@ void CDasherModel::SetContext(std::string &sNewContext) {
 
   m_Rootmin = GetLongParameter(LP_MAX_Y) / 2 - iWidth / 2;
   m_Rootmax = GetLongParameter(LP_MAX_Y) / 2 + iWidth / 2;
+
+  m_iTargetOffset = 0;
+
+//   m_iTargetMin = m_Rootmin;
+//   m_iTargetMax = m_Rootmax;
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -396,7 +426,8 @@ void CDasherModel::SetContext(std::string &sNewContext) {
 /// \return Returns the number of nats entered
 ///
 
-double CDasherModel::Get_new_root_coords(myint Mousex, myint Mousey) {
+void CDasherModel::Get_new_root_coords(myint Mousex, myint Mousey,
+                                         myint &iNewMin, myint &iNewMax) {
   // Comments refer to the code immedialtely before them
 
   if(Mousex <= 0) {
@@ -456,134 +487,67 @@ double CDasherModel::Get_new_root_coords(myint Mousex, myint Mousey) {
   // maximum zoom.
 
   if((iTargetMax - iTargetMin) < iMinSize) {
-
     iNewTargetMin = iTargetMin * (iMaxY - iMinSize) / (iMaxY - (iTargetMax - iTargetMin));
     iNewTargetMax = iNewTargetMin + iMinSize;
 
     iTargetMin = iNewTargetMin;
     iTargetMax = iNewTargetMax;
-
   }
 
-  // Check we're not going faster than the speed slider setting
-  // allows, and adjust if necessary. Note that if we re-size the
-  // target we need to be careful about where we centre the new range
-  // (hence the slightly complicated expression above)
-
-  DoZoom(iTargetMin, iTargetMax);
-
-  // Actually do the zooming
-
-  return -1.0 * log((iTargetMax - iTargetMin) / static_cast < double >(iMaxY));
-
-  // Return value is the zoom factor so we can keep track of bitrate.
+  iNewMin = (((m_Rootmin - iTargetMin) * (myint)GetLongParameter(LP_MAX_Y)) / (iTargetMax - iTargetMin));
+  iNewMax = (((m_Rootmax - iTargetMax) * (myint)GetLongParameter(LP_MAX_Y)) / (iTargetMax - iTargetMin) + (myint)GetLongParameter(LP_MAX_Y));
 }
 
-/// Zoom the display so that [iTargetMin,iTargetMax] (in current
-/// Dasher co-ordinates) are the new extremes of the viewport.
-
-void CDasherModel::DoZoom(myint iTargetMin, myint iTargetMax) {
-
-  // std::cout << "iTargetMin: " << iTargetMin << " iTargetMax: " << iTargetMax << std::endl;
-
-  myint newRootmin(((m_Rootmin - iTargetMin) * (myint)GetLongParameter(LP_MAX_Y)) / (iTargetMax - iTargetMin));
-  myint newRootmax(((m_Rootmax - iTargetMax) * (myint)GetLongParameter(LP_MAX_Y)) / (iTargetMax - iTargetMin) + (myint)GetLongParameter(LP_MAX_Y));
-
-  // Update the max and min of the root node to make iTargetMin and iTargetMax the edges of the viewport.
-
-  if(newRootmin > (myint)GetLongParameter(LP_MAX_Y) / 2 - 1)
-    newRootmin = (myint)GetLongParameter(LP_MAX_Y) / 2 - 1;
-
-  if(newRootmax < (myint)GetLongParameter(LP_MAX_Y) / 2 + 1)
-    newRootmax = (myint)GetLongParameter(LP_MAX_Y) / 2 + 1;
-
-  // Check that we haven't drifted too far. The rule is that we're not
-  // allowed to let the root max and min cross the midpoint of the
-  // screen.
-
-  if(newRootmax < m_Rootmax_max && newRootmin > m_Rootmin_min && (newRootmax - newRootmin) > (myint)GetLongParameter(LP_MAX_Y) / 4) {
-    // Only update if we're not making things big enough to risk
-    // overflow. In theory we should have reparented the root well
-    // before getting this far.
-    //
-    // Also don't allow the update if it will result in making the
-    // root too small. Again, we should have re-generated a deeper
-    // root in most cases, but the original root is an exception.
-
-    m_Rootmax = newRootmax;
-    m_Rootmin = newRootmin;
-  }
-  else {
-    // TODO - force a new root to be chosen, so that we get better
-    // behaviour than just having Dasher stop at this point.
-  }
-}
-
-void CDasherModel::Get_new_goto_coords(double zoomfactor, myint MouseY)
-                                       // this was mousex.
-{
-  // First, we need to work out how far we need to zoom in
-  //float zoomfactor=(m_DasherOX-MouseX)/(m_DasherOX*1.0);
-
-  // Then zoom in appropriately
-  m_Rootmax = m_Rootmax + myint(zoomfactor * (m_Rootmax - (myint)GetLongParameter(LP_MAX_Y) / 2));
-  m_Rootmin = m_Rootmin + myint(zoomfactor * (m_Rootmin - (myint)GetLongParameter(LP_MAX_Y) / 2));
-
-  // Afterwards, we need to take care of the vertical offset.
-  myint up = ((myint)GetLongParameter(LP_MAX_Y) / 2) - MouseY;
-  m_Rootmax = m_Rootmax + up;
-  m_Rootmin = m_Rootmin + up;
-}
-
-myint CDasherModel::PlotGoTo(myint MouseX, myint MouseY) {
-  // First, we need to work out how far we need to zoom in
-  double zoomfactor = ((myint)GetLongParameter(LP_OX) - MouseX) / ((myint)GetLongParameter(LP_OX) * 1.0);
-  zoomfactor = pow(0.5, zoomfactor);
-
-  myint height = int ((myint)GetLongParameter(LP_MAX_Y) * zoomfactor / 2);
-
-  return height;
-}
-
-void CDasherModel::Tap_on_display(myint miMousex,
+bool CDasherModel::Tap_on_display(myint miMousex,
                                   myint miMousey, 
                                   unsigned long Time, 
                                   Dasher::VECTOR_SYMBOL_PROB* pAdded, 
                                   int* pNumDeleted) 
         // work out the next viewpoint, opens some new nodes
 {
-
-  //  std::cout << "miMouseX: " << miMousex << " miMouseY: " << miMousey << std::endl;
-
-  if(!GetBoolParameter(BP_DASHER_PAUSED) && (m_deGotoQueue.size() == 0) ) {
-
+  // TODO: Reimplement this 
   // Clear out parameters that might get passed in to track user activity
   if (pAdded != NULL)
     pAdded->clear();
   if (pNumDeleted != NULL)
     *pNumDeleted = 0;
 
-  // Find out the current node under the crosshair
-  CDasherNode *old_under_cross = Get_node_under_crosshair();
+  if(GetBoolParameter(BP_DASHER_PAUSED) && (m_deGotoQueue.size() == 0))
+     return false;
 
-  // works out next viewpoint
-  total_nats += Get_new_root_coords(miMousex, miMousey);
+  myint iNewMin;
+  myint iNewMax;
+
+  if(m_deGotoQueue.size() == 0) {
+    // works out next viewpoint
+    Get_new_root_coords(miMousex, miMousey, iNewMin, iNewMax);
   
- 
+    if(GetBoolParameter(BP_OLD_STYLE_PUSH))
+      OldPush(miMousex, miMousey);
+  }
+  else {
+    iNewMin = m_deGotoQueue.front().iN1;
+    iNewMax = m_deGotoQueue.front().iN2;
+    m_deGotoQueue.pop_front();
+  }
 
-  // opens up new nodes
+  // Now actually zoom to the new location
+  NewGoTo(iNewMin, iNewMax);
 
-  if(GetBoolParameter(BP_OLD_STYLE_PUSH))
-  {
+  total_nats += -1.0 * log((iNewMax - iNewMin) / 4096.0);
 
+  return true;
+}
+
+void CDasherModel::OldPush(myint iMousex, myint iMousey) {
   // push node under mouse
-  CDasherNode *pUnderMouse = Get_node_under_mouse(miMousex, miMousey);
+  CDasherNode *pUnderMouse = Get_node_under_mouse(iMousex, iMousey);
 
   Push_Node(pUnderMouse);
 
   if(Framerate() > 4) {
     // push node under mouse but with x coord on RHS
-    CDasherNode *pRight = Get_node_under_mouse(50, miMousey);
+    CDasherNode *pRight = Get_node_under_mouse(50, iMousey);
     Push_Node(pRight);
   }
 
@@ -597,7 +561,7 @@ void CDasherModel::Tap_on_display(myint miMousex,
 
   if(Framerate() > 8) {
     // add some noise and push another node
-    CDasherNode *pRight = Get_node_under_mouse(50, miMousey + iRandom % 500 - 250);
+    CDasherNode *pRight = Get_node_under_mouse(50, iMousey + iRandom % 500 - 250);
     Push_Node(pRight);
   }
 
@@ -605,7 +569,7 @@ void CDasherModel::Tap_on_display(myint miMousex,
 
   if(Framerate() > 15) {
     // add some noise and push another node
-    CDasherNode *pRight = Get_node_under_mouse(50, miMousey + iRandom % 500 - 250);
+    CDasherNode *pRight = Get_node_under_mouse(50, iMousey + iRandom % 500 - 250);
     Push_Node(pRight);
   }
 
@@ -613,62 +577,23 @@ void CDasherModel::Tap_on_display(myint miMousex,
   if(Framerate() > 30) {
     for(int i = 1; i < int (Framerate() - 30) / 3; i++) {
 
+      int iRandom = RandomInt();
+      
+      if(Framerate() > 8) {
+	      // add some noise and push another node
+      	CDasherNode *pRight = Get_node_under_mouse(50, iMousey + iRandom % 500 - 250);
+	      Push_Node(pRight);
+      }
+      
       iRandom = RandomInt();
       // push at a random node on the RHS
-      CDasherNode *pRight = Get_node_under_mouse(50, miMousey + iRandom % 1000 - 500);
+      CDasherNode *pRight = Get_node_under_mouse(50, iMousey + iRandom % 1000 - 500);
       Push_Node(pRight);
 
     }
   }
-  }
-//      Update(m_Root,under_mouse,0);
-
-  CDasherNode *new_under_cross = Get_node_under_crosshair();
-
-  // FIXME - Reimplement
-
-  if(new_under_cross != old_under_cross) {
-    DeleteCharacters(new_under_cross, old_under_cross, pNumDeleted);
-  }
-
-  if(new_under_cross->isSeen() == true) {
-//     if(new_under_cross->ControlChild() != true) {
-//       SetBitrate(m_dMaxRate);
-//     }
-     return;
-   }
-
-
-
-
-  // FIXME - Need to recurse up possibly unseen parents
-
-
-  RecursiveOutput(new_under_cross, pAdded);
-
-  // FIXME - Reimplement
-
-//   if(new_under_cross->ControlChild() == true) {
-//     //            m_pEditbox->outputcontrol(new_under_cross->GetControlTree()->pointer,new_under_cross->GetControlTree()->data,new_under_cross->GetControlTree()->type);
-//     OutputCharacters(new_under_cross);
-//     SetBitrate(m_dMaxRate / 3);
-//   }
-//   else {
-//     OutputCharacters(new_under_cross);
-//     SetBitrate(m_dMaxRate);
-//   }
-  //      m_Root->Recursive_Push_Node(0);
-
-  }
-
-  if(m_deGotoQueue.size() > 0) {
-    NewGoTo(m_deGotoQueue.front().iN1, m_deGotoQueue.front().iN2, m_deGotoQueue.front().iStyle);
-    m_deGotoQueue.pop_front();
-  }
-
 }
 
-
 void CDasherModel::RecursiveOutput(CDasherNode *pNode, Dasher::VECTOR_SYMBOL_PROB* pAdded) {
   if(pNode->Parent() && (!pNode->Parent()->isSeen()))
     RecursiveOutput(pNode->Parent(), pAdded);
@@ -753,74 +678,80 @@ double CDasherModel::Plan_new_goto_coords(int iRxnew, myint mousey, int *iSteps,
   return iRxnew_log;
 }
 
-void CDasherModel::NewGoTo(myint n1 , myint n2 , int style) {
+void CDasherModel::NewGoTo(myint newRootmin, myint newRootmax) {
   // Find out the current node under the crosshair
   CDasherNode *old_under_cross=Get_node_under_crosshair();
 
-  // establish next viewpoint
-  m_Rootmin = n1 ;
-  m_Rootmax = n2 ;
-   
+  // Update the max and min of the root node to make iTargetMin and iTargetMax the edges of the viewport.
+
+  if(newRootmin > (myint)GetLongParameter(LP_MAX_Y) / 2 - 1)
+    newRootmin = (myint)GetLongParameter(LP_MAX_Y) / 2 - 1;
+
+  if(newRootmax < (myint)GetLongParameter(LP_MAX_Y) / 2 + 1)
+    newRootmax = (myint)GetLongParameter(LP_MAX_Y) / 2 + 1;
+
+  // Check that we haven't drifted too far. The rule is that we're not
+  // allowed to let the root max and min cross the midpoint of the
+  // screen.
+
+  if(newRootmax < m_Rootmax_max && newRootmin > m_Rootmin_min && (newRootmax - newRootmin) > (myint)GetLongParameter(LP_MAX_Y) / 4) {
+    // Only update if we're not making things big enough to risk
+    // overflow. In theory we should have reparented the root well
+    // before getting this far.
+    //
+    // Also don't allow the update if it will result in making the
+    // root too small. Again, we should have re-generated a deeper
+    // root in most cases, but the original root is an exception.
+
+    m_Rootmax = newRootmax;
+    m_Rootmin = newRootmin;
+
+    m_iTargetOffset = (m_iTargetOffset * 90) / 100;
+    
+    //     m_iTargetMax = m_iTargetMax + 0.1 * (m_Rootmax - m_iTargetMax);
+//     m_iTargetMin =  m_iTargetMin + 0.1 * (m_Rootmin - m_iTargetMin);
+  }
+  else {
+    // TODO - force a new root to be chosen, so that we get better
+    // behaviour than just having Dasher stop at this point.
+  }
+
   // push node under crosshair
   CDasherNode* new_under_cross = Get_node_under_crosshair();
   Push_Node(new_under_cross);
+
+  HandleOutput(new_under_cross, old_under_cross);
+
+ }
+
+void CDasherModel::HandleOutput(CDasherNode *pNewNode, CDasherNode *pOldNode) {
+  if(pNewNode != pOldNode)
+    DeleteCharacters(pNewNode, pOldNode);
   
-  if ( style >= 2 ) { // "2" means FINAL. "1" means STEP
-	// Ugly hack  --
-    // Push at all locations on the right hand side.
-    int Hack=60 ; // was 20
-    int hack ;
-    int y1 , y2 , hackystep = 34 , mid=GetLongParameter(LP_MAX_Y)/2 , top=GetLongParameter(LP_MAX_Y)*1/20 , bot = GetLongParameter(LP_MAX_Y)*19/20 ;
-    // y1 sweeps the top half of the screen; y2 sweeps the bottom half
-    // choice of `random' hackystep intended to give fairly uniform coverage. 
-    for ( y1 = mid , y2 = mid+hackystep/2 , hack = 1 ; hack <= Hack ; hack ++ ) {
-      // We don't have a mousex, so "emulating" one.
-      CDasherNode* node_under_goto = Get_node_under_mouse(50, y1);
-      Push_Node(node_under_goto);
-      node_under_goto = Get_node_under_mouse(50, y2);
-      Push_Node(node_under_goto);
-    
-      y1 -= hackystep;
-      if ( y1 < top ) { y1 += mid; }
-
-      y2 += hackystep;
-      if ( y1 > bot ) { y2 -= mid; }
-    }
-    // end hack
-  }
-  
-  //Update(m_Root,new_under_cross,0);
-
-  if (new_under_cross!=old_under_cross) {
-    DeleteCharacters(new_under_cross,old_under_cross);
-  }
-
-  if (new_under_cross->isSeen()==true)
+  if(pNewNode->isSeen())
     return;
 
-  new_under_cross->Seen(true);
-
-  OutputCharacters(new_under_cross);
+  // TODO: Reimplement second parameter
+  RecursiveOutput(pNewNode, NULL);
 }
 
 
 // TODO - is this used any more
-void CDasherModel::OutputCharacters(CDasherNode *node) {
-  if(node->Parent() != NULL && node->Parent()->isSeen() != true) {
-    node->Parent()->Seen(true);
-    OutputCharacters(node->Parent());
-  }
-  symbol t = node->Symbol();
-  if(t)                         // SYM0
-  {
-    Dasher::CEditEvent oEvent(1, GetAlphabet().GetText(t));
-    InsertEvent(&oEvent);
-  }
-}
+//void CDasherModel::OutputCharacters(CDasherNode *node) {
+//  if(node->Parent() != NULL && node->Parent()->isSeen() != true) {
+//    node->Parent()->Seen(true);
+//    OutputCharacters(node->Parent());
+//  }
+//  symbol t = node->Symbol();
+//  if(t)                         // SYM0
+//  {
+//    Dasher::CEditEvent oEvent(1, GetAlphabet().GetText(t));
+//    InsertEvent(&oEvent);
+//  }
+//}
+//
 
-// TODO - is this used any more?
 bool CDasherModel::DeleteCharacters(CDasherNode *newnode, CDasherNode *oldnode, int* pNumDeleted) {
-
   // DJW cant see how either of these can ever be NULL
   DASHER_ASSERT_VALIDPTR_RW(newnode);
   DASHER_ASSERT_VALIDPTR_RW(oldnode);
@@ -962,9 +893,9 @@ CDasherModel::CTrainer::CTrainer(CDasherModel &DasherModel)
   m_Context = m_DasherModel.m_pLanguageModel->CreateEmptyContext();
 }
 
-void CDasherModel::CTrainer::Train(const std::vector <symbol >&vSymbols) {
-  for(unsigned int i(0); i < vSymbols.size(); i++)
-    m_DasherModel.m_pLanguageModel->LearnSymbol(m_Context, vSymbols[i]);
+void CDasherModel::CTrainer::Train(const std::vector<symbol> &vSymbols) {
+  for(std::vector<symbol>::const_iterator it(vSymbols.begin()); it != vSymbols.end(); ++it)
+    m_DasherModel.m_pLanguageModel->LearnSymbol(m_Context, *it);
 }
 
 CDasherModel::CTrainer::~CTrainer() {
@@ -1014,7 +945,7 @@ void CDasherModel::Push_Node(CDasherNode *pNode) {
   }
 
   pNode->Alive(true);
-  
+
   pNode->m_pNodeManager->PopulateChildren(pNode);
   pNode->SetHasAllChildren(true);
 }
@@ -1025,9 +956,9 @@ void CDasherModel::Recursive_Push_Node(CDasherNode *pNode, int iDepth) {
     return;
   }
 
-  if(pNode->Symbol() == GetControlSymbol()) {
-    return;
-  }
+ //  if(pNode->Symbol() == GetControlSymbol()) {
+//     return;
+//   }
 
   Push_Node(pNode);
 
@@ -1062,7 +993,10 @@ bool CDasherModel::RenderToView(CDasherView *pView, bool bRedrawDisplay) {
 
   bool bReturnValue;
 
-  bReturnValue = pView->Render(m_Root, m_Rootmin, m_Rootmax, vNodeList, vDeleteList, bRedrawDisplay);
+  //if(GetBoolParameter(BP_DELAY_VIEW))
+  //  bReturnValue = pView->Render(m_Root, m_iTargetMin, m_iTargetMax, vNodeList, vDeleteList, bRedrawDisplay, m_bGameMode);
+  //else
+    bReturnValue = pView->Render(m_Root, m_Rootmin + m_iTargetOffset, m_Rootmax + m_iTargetOffset, vNodeList, vDeleteList, bRedrawDisplay, m_bGameMode);
 
   if(!GetBoolParameter(BP_OLD_STYLE_PUSH)) {
   for(std::vector<CDasherNode *>::iterator it(vNodeList.begin()); it != vNodeList.end(); ++it)
@@ -1077,8 +1011,8 @@ bool CDasherModel::RenderToView(CDasherView *pView, bool bRedrawDisplay) {
 
 bool CDasherModel::CheckForNewRoot(CDasherView *pView) {
 
-  if(m_deGotoQueue.size() > 0)
-    return false;
+//  if(m_deGotoQueue.size() > 0)
+ //   return false;
 
   CDasherNode *root(m_Root);
   CDasherNode::ChildMap & children = m_Root->Children();
@@ -1125,40 +1059,50 @@ bool CDasherModel::CheckForNewRoot(CDasherView *pView) {
   return false;
 }
 
+double CDasherModel::CorrectionFactor(int dasherx, int dashery) {
+  double dX = 1 - dasherx/2048.0;
+  double dY = dashery/2048.0 - 1;
+
+  double dR = sqrt(pow(dX, 2.0) + pow(dY, 2.0));
+
+  if(fabs(dX) < 0.1)
+    return dR * (1 + dX /2.0+ pow(dX, 2.0) / 3.0 + pow(dX, 3.0) / 4.0 + pow(dX, 4.0) / 5.0);
+  else
+    return -dR * log(1 - dX) / dX;
+}
+
 void CDasherModel::ScheduleZoom(int dasherx, int dashery) {
+
+   // TODO: What is the following line for?
    if (dasherx < 2) { dasherx = 100; }
-   int iSteps = GetLongParameter(LP_ZOOMSTEPS); 
-   myint iRxnew = ((GetLongParameter(LP_OX)/2) * GetLongParameter(LP_OX)) / dasherx;
-   myint o1, o2, n1, n2;
-   Plan_new_goto_coords(iRxnew, dashery, &iSteps, &o1,&o2,&n1,&n2);
-   int s ;
-   myint rootMin, rootMax;
-   rootMin = n1;  
-   rootMax = n2;
-   myint zoomstep1, zoomstep2;
 
-   std::vector<CDasherNode *> vNodeList;
-   std::vector<CDasherNode *> vDeleteList;
+   double dCFactor(CorrectionFactor(dasherx, dashery));
 
+   int iSteps = GetLongParameter(LP_ZOOMSTEPS) * dCFactor;
+  // myint iRxnew = ((GetLongParameter(LP_OX)/2) * GetLongParameter(LP_OX)) / dasherx;
+   myint n1, n2, iTarget1, iTarget2;
+   //Plan_new_goto_coords(iRxnew, dashery, &iSteps, &o1,&o2,&n1,&n2);
+ 
+   iTarget1 = dashery - dasherx;
+   iTarget2 = dashery + dasherx;
+
+   double dZ = 4096 / static_cast<double>(iTarget2 - iTarget1);
+
+   n1 = (m_Rootmin - iTarget1) * dZ;
+   n2 = (m_Rootmax - iTarget2) * dZ + 4096;
+ 
    m_deGotoQueue.clear();
-
    SGotoItem sNewItem;
 
-   for (s = 1 ; s <= iSteps ; s ++) {
+   for(int s(1); s < iSteps; ++s) {
      // use simple linear interpolation. Really should do logarithmic interpolation, but
      // this will probably look fine.
-     zoomstep1 = (s * n1 + (iSteps-s) * o1) / iSteps;
-     zoomstep2 = (s * n2 + (iSteps-s) * o2) / iSteps;
-
-     sNewItem.iN1 = zoomstep1;
-     sNewItem.iN2 = zoomstep2;
+    
+     sNewItem.iN1 = (s * n1 + (iSteps-s) * m_Rootmin) / iSteps;
+     sNewItem.iN2 = (s * n2 + (iSteps-s) * m_Rootmax) / iSteps;
      sNewItem.iStyle = 1;
 
      m_deGotoQueue.push_back(sNewItem);
-
-   //   NewGoTo(zoomstep1, zoomstep2, 1);
-//      pView->Render(m_Root, m_Rootmin, m_Rootmax, vNodeList, vDeleteList);
-//      pView->Display();
    } 
 
    sNewItem.iN1 = n1;
@@ -1166,10 +1110,29 @@ void CDasherModel::ScheduleZoom(int dasherx, int dashery) {
    sNewItem.iStyle = 2;
  
    m_deGotoQueue.push_back(sNewItem);
+}
 
-   // NewGoTo(n1, n2, 2);
-//    pView->Render(m_Root, m_Rootmin, m_Rootmax, vNodeList, vDeleteList);
-//    pView->Display();
+void CDasherModel::Offset(int iOffset) {
+  m_Rootmin += iOffset;
+  m_Rootmax += iOffset;
 
-//    CheckForNewRoot(pView);
+  m_iTargetOffset -= iOffset;
+} 
+
+void CDasherModel::MatchTarget(bool bReverse) {
+  // TODO: Does anything need to happen wrt bReverse here?
+
+  m_Rootmin += m_iTargetOffset;
+  m_Rootmax += m_iTargetOffset;
+
+  m_iTargetOffset = 0;
+
+//   if(bReverse) {
+//     m_iTargetMin = m_Rootmin;
+//     m_iTargetMax = m_Rootmax;
+//   }
+//   else {
+//     m_Rootmin = m_iTargetMin;
+//     m_Rootmax = m_iTargetMax;
+//   }
 }
