@@ -78,12 +78,12 @@ public:
   /// Renders Dasher with mouse-dependent items
   /// \todo Clarify relationship between Render functions and probably only expose one
 
-  virtual bool Render(CDasherNode *pRoot, myint iRootMin, myint iRootMax, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList, bool bRedrawDisplay);
+  virtual bool Render(CDasherNode *pRoot, myint iRootMin, myint iRootMax, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList, bool bRedrawDisplay, bool bGameMode);
 
   /// Renders the Dasher node structure
   /// \todo Shouldn't be public?
 
-  virtual void RenderNodes(CDasherNode *pRoot, myint iRootMin, myint iRootMax, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList) = 0;
+  virtual void RenderNodes(CDasherNode *pRoot, myint iRootMin, myint iRootMax, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList, myint *iGamePointer) = 0;
 
   /// Translates the screen coordinates to Dasher coordinates and calls
   /// dashermodel.TapOnDisplay
@@ -126,7 +126,7 @@ public:
 
   /// Draw the game mode pointer - this shouldn't be here
 
-  virtual void DrawGameModePointer() = 0;
+  virtual void DrawGameModePointer(myint iPosition) = 0;
 
   ////// Return a reference to the screen
   
@@ -212,7 +212,8 @@ protected:
   // Orientation of Dasher Screen
   inline void MapScreen(screenint * DrawX, screenint * DrawY);
   inline void UnMapScreen(screenint * DrawX, screenint * DrawY);
-
+  bool m_bVisibleRegionValid;
+ 
  
   CDelayedDraw *m_pDelayDraw;
 
