@@ -85,7 +85,7 @@ void CDasherViewSquare::HandleEvent(Dasher::CEvent *pEvent) {
   }
 }
 
-void CDasherViewSquare::RenderNodes(CDasherNode *pRoot, myint iRootMin, myint iRootMax, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList) {
+void CDasherViewSquare::RenderNodes(CDasherNode *pRoot, myint iRootMin, myint iRootMax, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList, myint *iGamePointer) {
 
   DASHER_ASSERT(pRoot != 0);
 
@@ -97,7 +97,7 @@ void CDasherViewSquare::RenderNodes(CDasherNode *pRoot, myint iRootMin, myint iR
   myint iDasherMaxY;
   VisibleRegion(iDasherMinX, iDasherMinY, iDasherMaxX, iDasherMaxY);
  
-  RecursiveRender(pRoot, iRootMin, iRootMax, iDasherMaxX, vNodeList, vDeleteList);
+  RecursiveRender(pRoot, iRootMin, iRootMax, iDasherMaxX, vNodeList, vDeleteList, iGamePointer, true);
 
   // DelayDraw the text nodes
   m_pDelayDraw->Draw(Screen());
@@ -106,10 +106,14 @@ void CDasherViewSquare::RenderNodes(CDasherNode *pRoot, myint iRootMin, myint iR
 }
 
 
-int CDasherViewSquare::RecursiveRender(CDasherNode *pRender, myint y1, myint y2, int mostleft, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList) {
+int CDasherViewSquare::RecursiveRender(CDasherNode *pRender, myint y1, myint y2, int mostleft, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList, myint *iGamePointer, bool bDraw) {
   DASHER_ASSERT_VALIDPTR_RW(pRender);
 
-  if(!RenderNode(pRender->Colour(), pRender->ColorScheme(), y1, y2, mostleft, pRender->m_strDisplayText, pRender->m_bShove)) {
+  // TODO: We need an overhall of the node creation/deletion logic - make sure that we only maintain the minimum number of nodes which are actually needed.
+  // This is especially true at the moment in Game mode, which feels very sluggish. Node creation also feels slower in Windows than Linux, especially
+  // if many nodes are created at once (eg untrained Hiragana)
+
+  if(bDraw && !RenderNode(pRender->Colour(), y1, y2, mostleft, pRender->m_strDisplayText, pRender->m_bShove) && !(pRender->GetGame())) {
     vDeleteList.push_back(pRender);
     pRender->Kill();
     return 0;
@@ -120,6 +124,9 @@ int CDasherViewSquare::RecursiveRender(CDasherNode *pRender, myint y1, myint y2,
     return 0;
   }
 
+  if(pRender->GetGame())
+    *iGamePointer = (y1 + y2) / 2;
+  
   // Render groups
   RenderGroups(pRender, y1, y2, mostleft);
 
@@ -138,7 +145,10 @@ int CDasherViewSquare::RecursiveRender(CDasherNode *pRender, myint y1, myint y2,
     
     if((newy2 - newy1 > 50) || (pChild->Alive())) {
       pChild->Alive(true);
-      RecursiveRender(pChild, newy1, newy2, mostleft, vNodeList, vDeleteList);
+      RecursiveRender(pChild, newy1, newy2, mostleft, vNodeList, vDeleteList, iGamePointer, true);
+    }
+    else if(pRender->GetGame()) {
+      RecursiveRender(pChild, newy1, newy2, mostleft, vNodeList, vDeleteList, iGamePointer, false);
     }
   }
   
@@ -168,7 +178,7 @@ void CDasherViewSquare::RecursiveRenderGroups(SGroupInfo *pCurrentGroup, CDasher
     myint newy1 = y1 + (range * lbnd) / (int)GetLongParameter(LP_NORMALIZATION);
     myint newy2 = y1 + (range * hbnd) / (int)GetLongParameter(LP_NORMALIZATION);
     
-    RenderNode(pCurrentGroup->iColour, Opts::Groups, newy1, newy2, mostleft, pCurrentGroup->strLabel, true);
+    RenderNode(pCurrentGroup->iColour, newy1, newy2, mostleft, pCurrentGroup->strLabel, true);
   }
   
   // Iterate through child groups
@@ -193,7 +203,7 @@ CDasherViewSquare::Cymap::Cymap(myint iScale) {
 
 
 
-int CDasherViewSquare::RenderNode(const int Color, Opts::ColorSchemes ColorScheme, myint y1, myint y2, int &mostleft, const std::string &sDisplayText, bool bShove) {
+int CDasherViewSquare::RenderNode(const int Color, myint y1, myint y2, int &mostleft, const std::string &sDisplayText, bool bShove) {
 
   // Commenting because click mode occasionally fails this assert.
   // I don't know why.  -- cjb.
@@ -228,7 +238,7 @@ int CDasherViewSquare::RenderNode(const int Color, Opts::ColorSchemes ColorSchem
   int iTruncationType(GetLongParameter(LP_TRUNCATIONTYPE));
 
   if(iTruncation == 0) {        // Regular squares
-    DasherDrawRectangle(std::min(iDasherSize,iDasherMaxX), std::min(y2,iDasherMaxY), 0, std::max(y1,iDasherMinY), Color, -1, ColorScheme, GetBoolParameter(BP_OUTLINE_MODE), true, 1);
+    DasherDrawRectangle(std::min(iDasherSize,iDasherMaxX), std::min(y2,iDasherMaxY), 0, std::max(y1,iDasherMinY), Color, -1, Nodes1, GetBoolParameter(BP_OUTLINE_MODE), true, 1);
   }
   else {
     int iDasherY((myint)GetLongParameter(LP_MAX_Y));
@@ -677,6 +687,8 @@ void CDasherViewSquare::Input2Dasher(screenint iInputX, screenint iInputY, myint
 
   // Apply y scaling
 
+  // TODO: Check that this is still doing something vaguely sensible - I think it isn't
+
   if(iMode == 1 ) {
     if( GetLongParameter(LP_YSCALE) > 0 ) {
       
@@ -754,16 +766,13 @@ void CDasherViewSquare::TapOnDisplay(screenint mousex,
   }
 
 
+  // TODO: Mode probably isn't being used any more
+
   // Convert the input co-ordinates to dasher co-ordinates
 
   int mode;
   
-  if(GetBoolParameter(BP_NUMBER_DIMENSIONS))
-    mode = 1;
-  else if(GetBoolParameter(BP_EYETRACKER_MODE))
-    mode = 2;
-  else
-    mode = 0;
+  mode = 0;
  
   Input2Dasher(mousex, mousey, iDasherX, iDasherY, iType, mode);
   m_iDasherXCache = iDasherX;
@@ -805,6 +814,8 @@ void CDasherViewSquare::NewDrawGoTo(myint iDasherMin, myint iDasherMax, bool bAc
   Screen()->Polyline(p, 4, iWidth, iColour);
 }
 
+// TODO: Autocalibration should be in the eyetracker filter class
+
 void CDasherViewSquare::ResetSum() {
   m_ySum = 0;
 }
@@ -882,22 +893,105 @@ void CDasherViewSquare::AutoCalibrate(screenint *mousex, screenint *mousey) {
 
 // TODO - should be elsewhere
 
-void CDasherViewSquare::DrawGameModePointer() {
+void CDasherViewSquare::DrawGameModePointer(myint iPosition) {
+  // TODO: Horrible code duplication here
 
-  // FIXME - reimplement
+  if(iPosition > (myint)GetLongParameter(LP_MAX_Y)) {
+     CDasherScreen::point p[2];
+  
+  myint iDasherX;
+  myint iDasherY;
+  
+  iDasherX = -400;
+  iDasherY = GetLongParameter(LP_MAX_Y);
+  
+  Dasher2Screen(iDasherX, iDasherY, p[0].x, p[0].y);
+  
+  iDasherX = -400;
+  iDasherY = GetLongParameter(LP_MAX_Y) - 500;
+  
+  Dasher2Screen(iDasherX, iDasherY, p[1].x, p[1].y);
+  
+  Screen()->Polyline(p, 2, 1, 2);
 
-//   myint loc = DasherModel()->GetGameModePointerLoc();
+   iDasherX = -300;
+  iDasherY = GetLongParameter(LP_MAX_Y) - 100;
+  
+  Dasher2Screen(iDasherX, iDasherY, p[1].x, p[1].y);
+  
+  Screen()->Polyline(p, 2, 1, 2);
 
-//   if(loc == myint(INT64_MIN))
-//     return;
+   iDasherX = -500;
+  iDasherY = GetLongParameter(LP_MAX_Y) - 100;
+  
+  Dasher2Screen(iDasherX, iDasherY, p[1].x, p[1].y);
+  
+  Screen()->Polyline(p, 2, 1, 2);
 
-//   if(loc > GetLongParameter(LP_MAX_Y))
-//     DasherDrawCentredRectangle(-50, GetLongParameter(LP_MAX_Y), 5, 135, Opts::ColorSchemes(Objects), false);
+  }
+  else if(iPosition < 0) {
+         CDasherScreen::point p[2];
+  
+  myint iDasherX;
+  myint iDasherY;
+  
+  iDasherX = -400;
+  iDasherY = 0;
+  
+  Dasher2Screen(iDasherX, iDasherY, p[0].x, p[0].y);
+  
+  iDasherX = -400;
+  iDasherY = 500;
+  
+  Dasher2Screen(iDasherX, iDasherY, p[1].x, p[1].y);
+  
+  Screen()->Polyline(p, 2, 1, 2);
 
-//   else if(loc < 0)
-//     DasherDrawCentredRectangle(-50, 0, 5, 135, Opts::ColorSchemes(Objects), false);
+   iDasherX = -300;
+  iDasherY = 100;
+  
+  Dasher2Screen(iDasherX, iDasherY, p[1].x, p[1].y);
+  
+  Screen()->Polyline(p, 2, 1, 2);
 
-//   else
-//     DasherDrawCentredRectangle(-50, loc, 7, 135, Opts::ColorSchemes(Objects), false);
+   iDasherX = -500;
+  iDasherY = 100;
+  
+  Dasher2Screen(iDasherX, iDasherY, p[1].x, p[1].y);
+  
+  Screen()->Polyline(p, 2, 1, 2);
+  }
+  else {
+ CDasherScreen::point p[2];
+  
+  myint iDasherX;
+  myint iDasherY;
+  
+  iDasherX = -200;
+  iDasherY = iPosition;
+  
+  Dasher2Screen(iDasherX, iDasherY, p[0].x, p[0].y);
+  
+  iDasherX = -1000;
+  iDasherY = iPosition;
+  
+  Dasher2Screen(iDasherX, iDasherY, p[1].x, p[1].y);
+  
+  Screen()->Polyline(p, 2, 1, 2);
+
+   iDasherX = -300;
+  iDasherY = iPosition + 100;
+  
+  Dasher2Screen(iDasherX, iDasherY, p[1].x, p[1].y);
+  
+  Screen()->Polyline(p, 2, 1, 2);
+
+   iDasherX = -300;
+  iDasherY = iPosition - 100;
+  
+  Dasher2Screen(iDasherX, iDasherY, p[1].x, p[1].y);
+  
+  Screen()->Polyline(p, 2, 1, 2);
+   }
 
 }
